//------------------------------------------------------------
//    knob.cpp
//------------------------------------------------------------
#include "knob.h"
#include "mouse_stat.h"
#include <math.h>

//------------------------------------------------------------
//    const
//------------------------------------------------------------
const unsigned int default_radius = 40;
const unsigned int default_divcount = 10;
const double default_min_angle = M_PI / 6.0;
const double default_max_angle = M_PI * (2.0 - 1.0 / 6.0);
const wxColour default_line_colour = *wxBLACK;
const wxColour default_knob_colour = *wxLIGHT_GREY;
const unsigned int knob_size[2] = {300, 300};
const wxEventType myEVT_KNOB_UPDATED = wxNewEventType();
//------------------------------------------------------------
//    
//------------------------------------------------------------
Knob::Knob(wxWindow* parent, wxWindowID id,
	double val1, double min1, double max1,
	int radius1, long style)
{
	if(Create(parent, id, val1, min1, max1, radius1, wxDefaultPosition, wxDefaultSize, style))
	{
		mouse_1 = new MouseStatus();
		Connect(wxEVT_PAINT, wxPaintEventHandler(Knob::OnPaint));
		Connect(wxEVT_SIZE, wxSizeEventHandler(Knob::OnSize));
		Connect(wxEVT_LEFT_DOWN, wxMouseEventHandler(Knob::OnMouse));
		Connect(wxEVT_LEFT_UP, wxMouseEventHandler(Knob::OnMouse));
		Connect(wxEVT_MOTION, wxMouseEventHandler(Knob::OnMouseMove));
	}
}

//------------------------------------------------------------
//    
//------------------------------------------------------------
bool Knob::Create(wxWindow* parent, wxWindowID id,
	double val1, double min1, double max1, int radius1,
	const wxPoint& pos, const wxSize& size, long style)
{
	min_angle = default_min_angle;
	max_angle = default_max_angle;
	val = val1;
	min = min1;
	max = max1;
	angle = (val - min) * (max_angle - min_angle) / (max - min) + min_angle;
	xorig = size.GetWidth() / 2;
	yorig = size.GetHeight() / 2;
	radius = radius1;
	if(!wxControl::Create(parent, id, pos, size, style)) return false;
	SetScaleDivision(default_divcount);
	SetRange(min1, max1);
	SetValue(val1);
	return true;
}

wxSize Knob::DoGetBestSize() const
{
	int l1 = radius * 2.8;
	return wxSize(l1, l1);
}

//------------------------------------------------------------
//    
//------------------------------------------------------------
Knob::~Knob()
{
	delete mouse_1;
}

//------------------------------------------------------------
//    
//------------------------------------------------------------
void Knob::OnPaint(wxPaintEvent& event)
{
	Render();
}

//------------------------------------------------------------
//    
//------------------------------------------------------------
void Knob::OnSize(wxSizeEvent& event)
{
	wxSize size1 = GetSize();
	int x1 = size1.GetWidth() / 2;
	int y1 = size1.GetHeight() / 2;
	xorig = x1;
	yorig = y1;
	UpdateScale();
	Refresh(true);
}


//------------------------------------------------------------
//    
//------------------------------------------------------------
void Knob::OnMouse(wxMouseEvent& event)
{
	int l1 = (xorig - event.GetX()) * (xorig - event.GetX())
		+ (yorig - event.GetY()) * (yorig - event.GetY());
	bool b1 = (l1 <= (radius * radius));
	if(event.LeftDown() && b1){
		mouse_1->b_1 = LEFT;
		mouse_1->pre_pos[0] = (int)event.GetX();
		mouse_1->pre_pos[1] = (int)event.GetY();
	}
	if(event.LeftUp()){
		mouse_1->b_1 = NON;
	}
}


//------------------------------------------------------------
//    
//------------------------------------------------------------
void Knob::OnMouseMove(wxMouseEvent& event)
{
	if(mouse_1->b_1 == LEFT){
		mouse_1->pos[0] = (int)event.GetX();
		mouse_1->pos[1] = (int)event.GetY();
		int v1x = mouse_1->pre_pos[0] - xorig;
		int v1y = mouse_1->pre_pos[1] - yorig;
		int v2x = mouse_1->pos[0] - xorig;
		int v2y = mouse_1->pos[1] - yorig;
		int c1 = v1x * v2y - v1y * v2x;
		int c2 = v1x * v2x + v1y * v2y;
		mouse_1->pre_pos[0] = mouse_1->pos[0];
		mouse_1->pre_pos[1] = mouse_1->pos[1];
		double angle1;
		if(c1==0) return;
		else if((c2==0) && (c1>0)) angle1 = 0.5 * M_PI + angle;
		else if((c2==0) && (c1<0)) angle1 = - 0.5 * M_PI + angle;
		else angle1 = atan2((double)c1, (double)c2) + angle;
		
		if((angle1 >= min_angle) && (angle1 <= max_angle)){
			angle = angle1;
			double val1 = min + (max - min) * (angle - min_angle) / (max_angle - min_angle);
			SetValue(val1);
			Refresh(false);
			Update();
		}else if((angle1 < min_angle) && (angle != min_angle)){
			SetValue(min);
			Refresh(false);
			Update();
		}else if((angle1 > max_angle) && (angle != max_angle)){
			SetValue(max);
			Refresh(false);
			Update();
		}
	}
}

//------------------------------------------------------------
//    
//------------------------------------------------------------
void Knob::Render()
{
	wxPaintDC dc1(this);
	double c1 = angle;
	double px = - ((double)radius * 0.8) * sin(c1) + (double)xorig;
	double py = ((double)radius * 0.8) * cos(c1) + (double)yorig;
	dc1.DrawCircle(xorig, yorig, radius);
	for(int i1 = 0, sz = lines.size(); i1 < sz; i1++){
		dc1.DrawLine(lines[i1][0], lines[i1][1], lines[i1][2], lines[i1][3]);
	}
	dc1.DrawCircle((int)px, (int)py, (int)radius * 0.1);
}

//------------------------------------------------------------
//    
//------------------------------------------------------------
void Knob::SetScaleDivision(int div1)
{
	if(div1 < 2){return;}
	divcount = div1;
	double angle_range = max_angle - min_angle;
	double div_angle = (max_angle - min_angle) / double(div1);
	double length1 = (double)radius * 0.25;
	double space1 = (double)radius * 0.1;
	lines.assign(div1+1, std::vector<int>(4, 0));
	double ang1 = min_angle;
	double l1 = length1;
	double p1x, p1y, p2x, p2y;
	for(int i2 = 0; i2 < divcount + 1; i2++){
		p1x = (double)xorig - ((double)radius + space1) * sin(ang1);
		p1y = (double)yorig + ((double)radius + space1) * cos(ang1);
		p2x = (double)xorig - ((double)radius + space1 + l1) * sin(ang1);
		p2y = (double)yorig + ((double)radius + space1 + l1) * cos(ang1);
		lines[i2][0] = (int)p1x;
		lines[i2][1] = (int)p1y;
		lines[i2][2] = (int)p2x;
		lines[i2][3] = (int)p2y;
		ang1 += div_angle;
	}
}

void Knob::UpdateScale()
{
	double angle_range = max_angle - min_angle;
	double div_angle = angle_range / (double)divcount;
	double length1 = (double)radius * 0.25;
	double space1 = (double)radius * 0.1;
	lines.assign(divcount+1, std::vector<int>(4, 0));
	double ang1 = min_angle;
	double l1 = length1;
	double p1x, p1y, p2x, p2y;
	for(int i2 = 0; i2 < divcount + 1; i2++){
		p1x = (double)xorig - ((double)radius + space1) * sin(ang1);
		p1y = (double)yorig + ((double)radius + space1) * cos(ang1);
		p2x = (double)xorig - ((double)radius + space1 + l1) * sin(ang1);
		p2y = (double)yorig + ((double)radius + space1 + l1) * cos(ang1);
		lines[i2][0] = (int)p1x;
		lines[i2][1] = (int)p1y;
		lines[i2][2] = (int)p2x;
		lines[i2][3] = (int)p2y;
		ang1 += div_angle;
	}
}

//------------------------------------------------------------
//    
//------------------------------------------------------------
void Knob::SetKnobRadius(const int r1)
{
	if(r1 < 10) return;
	radius = r1;
	UpdateScale();
	Refresh(true);
}

//------------------------------------------------------------
//    
//------------------------------------------------------------
void Knob::SetValue(const double v1)
{
	if((v1 >= min) && (v1 <= max))
	{
		val = v1;
		wxCommandEvent event(myEVT_KNOB_UPDATED);
		event.SetId(GetId());
		wxPostEvent(this, event);
	}
}
